@@ -676,6 +676,10 @@ class Pd2d(DataN):
         ls_out.extend([_.to_cif(separator=separator)+"\n" for _ in l_loopn])
         return "\n".join(ls_out)
 
+    def apply_constraints(self):
+        """Apply constraints."""
+        pass
+
 # s_cont = """
 #  data_powder2d
 #  _setup_wavelength     0.840
